<?php

namespace Drupal\notification_message\Entity;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityPublishedInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Define the notification message access control handler.
 */
class NotificationMessageAccess extends EntityAccessControlHandler {

  /**
   * {@inheritDoc}
   */
  protected function checkAccess(
    EntityInterface $entity,
    $operation,
    AccountInterface $account
  ) {
    $admin_permission = $entity->getEntityType()->getAdminPermission();
    if ($account->hasPermission($admin_permission)) {
      return AccessResult::allowedIfHasPermission($account, $admin_permission);
    }
    if ($operation === 'view') {
      if ($entity instanceof EntityPublishedInterface) {
        $allowed_if_published = AccessResult::allowedIf($entity->isPublished())->addCacheableDependency($entity);
        $allowed_if_view_any_unpublished = AccessResult::allowedIfHasPermission($account, 'view any unpublished notification message');
        $is_owner = $entity->getAuthorUser()->id() == $account->id();
        $has_view_own_permission = AccessResult::allowedIfHasPermission($account, 'view own unpublished notification message');
        $allowed_if_view_own_unpublished = AccessResult::allowedIf($is_owner)->addCacheableDependency($entity)->andIf($has_view_own_permission);
        return $allowed_if_published
          ->orIf($allowed_if_view_any_unpublished)
          ->orIf($allowed_if_view_own_unpublished);
      }
    }
    return AccessResult::neutral();
  }
}
